
# q-expansion of newform 5950.2.a.i, downloaded from the LMFDB on 17 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5950
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [2], [0], [-1], [-6], [2], [1], [0], [4], [8], [0], [-4], [-2], [8], [8], [6], [-4], [-8], [16], [4], [-10], [-12], [-12], [10], [-6], [6], [8], [2], [-8], [14], [16], [2], [10], [2], [22], [16], [14], [-22], [16], [20], [12], [0], [0], [-14], [12], [-4], [2], [0], [-10], [26], [14], [8], [22], [12], [18], [-16], [12], [-24], [-8], [-10], [18], [30], [8], [-16], [-6], [-4], [0], [-2], [-6], [-2], [-30], [16], [12], [14], [10], [-8], [-18], [-8], [6], [-10], [-38], [-6], [-12], [22], [-28], [-32], [30], [-10], [-18], [16], [-32], [-28], [-4], [-12], [-6], [36], [2], [30], [-12], [-28], [-14], [-26], [-12], [18], [-18], [-22], [8], [30], [8], [-14], [8], [10], [6], [10], [24], [22], [-14], [8], [16], [12], [-18], [26], [8], [-18], [-10], [-14], [40], [28], [0], [-6], [-36], [-16], [-36], [42], [-42], [46], [-42], [-10], [18], [22], [-2], [4], [-12], [30], [-46], [-24], [4], [-22], [36], [48], [8], [-30], [40], [-36], [-2], [8], [-16], [-18], [34], [-56], [18], [-34], [48], [-12], [38], [-12], [-12], [-32], [22], [-24], [12], [-2], [24], [-26], [8], [34], [26], [-36], [-40], [-6], [-32], [-14], [-52], [54], [-48], [30], [36], [32], [38], [-48], [54], [-46], [-12], [42], [30], [46], [-30], [44], [46], [-32], [-12], [28], [-10], [14], [40], [-2], [-48], [12], [-14], [48], [-46], [10], [-16], [0], [24], [14], [8], [42], [16], [6], [-2], [16], [-18], [32], [-60], [-6], [-34], [28], [-40], [-42], [-50], [54], [-56], [-70], [28], [-8], [70], [-44], [70], [16], [66], [-56], [-56], [-74], [14], [24], [-64], [-38], [16], [-24], [-6], [18], [48], [-70], [22], [-16], [-52], [58], [-24], [-22], [-16], [-76], [-32], [68], [42], [-28], [66], [-26], [2], [-6], [20], [-12], [-26], [-64], [-54], [-72], [32], [10], [10], [28], [64], [-16], [40], [22], [-2], [80], [34], [20], [-40], [34], [48], [-46], [66], [-10], [-28], [-36], [24], [66], [-38], [-44], [-74], [-78], [16], [-26], [-38], [50], [36], [56], [8], [26], [-4], [16], [30], [-68], [-80], [30], [16], [-48], [46], [86], [-6], [10], [-54], [0], [82], [18], [70], [74], [12], [-4], [-56], [84], [4], [8], [30], [-30], [2], [54], [34], [-88], [6], [94], [-48], [64], [30], [-74], [88], [-8], [-16], [72], [44], [2], [-18], [32], [38], [-14], [-40], [-74], [-38], [-64], [-92], [74], [16], [94], [72], [74], [12], [16], [62], [-32], [-34], [-12], [-30], [-32], [-32], [30], [56], [18], [-34], [14], [4], [26], [-60], [62], [74], [-24], [8], [78], [-34], [-96], [10], [72], [12], [-72], [48], [-86], [-32], [-58], [42], [90], [-32], [18], [72], [-54], [34], [-88], [34], [-50], [-36], [-42], [6], [50], [-4], [-18], [82], [-40], [36], [-8], [26], [16], [106], [20], [36], [-8], [-94], [-82], [58], [2], [-4], [72]]
